pub enum UpdateControlFlow {
Commit,
Abort,
}Expand description
The control flow corresponding to the atomic update output.
Variants
Commit
The update output value indicates that the atomic update has been committed.
This means try_open_atomic_update will consume the atomic update (i.e. return Ok(())),
and the atomic function call has to break.
Abort
The update output value indicates that the atomic update has been aborted.
This means try_open_atomic_update will give back the atomic update (i.e. return Err(Tracked(au))),
and the atomic function call has to continue.
Implementations